@@ -12,7 +12,7 @@ import os
 import wx
 
 import libtovid
-from libtovid import Group
+from libtovid.author import Group
 from libtovid.gui.constants import *
 from libtovid.gui.configs import TovidConfig
 from libtovid.gui.util import _
@@ -42,7 +42,7 @@ class DiscOptions:
         # Get global configuration (for output directory)
         curConfig = TovidConfig()
 
-        strCommand = "makexml -noask -quiet -overwrite -%s " % self.format
+        strCommand = "makexml -quiet -overwrite -%s " % self.format
 
         for curItem in self.optionList:
             # Prefix -topmenu or -menu if necessary
@@ -779,7 +779,7 @@ class VideoOptions:
 
         strCommand += "-in \"%s\" " % self.inFile
         strCommand += "-out \"%s/%s\" " % (curConfig.strOutputDirectory, self.outPrefix)
-        strCommand += "-from-gui"
+        strCommand += "-from-gui -noask"
         return strCommand
 
     def SetDiscFormat(self, format):
