@@ -71,7 +71,8 @@ __all__ = [
     'save_pdf',
     'save_png',
     'save_ps',
-    'save_svg']
+    'save_svg',
+    'write_ppm']
 
 import os
 import sys
@@ -82,6 +83,7 @@ from math import pi, sqrt
 import cairo
 import Image      # for JPG export
 import ImageColor # for getrgb, getcolor
+import ImageFile  # for write_png()
 from libtovid import log
 from libtovid.utils import to_unicode
 
@@ -154,6 +156,8 @@ class Drawing:
                            drawing operation
         """
         self.size = (width, height)
+        self.w = width
+        self.h = height
         self.aspect = float(width) / height
         self.autodraw = autodraw
         #self.commands = []
@@ -306,7 +310,8 @@ class Drawing:
         """Fill the current (closed) path with an optionally given color.
 
         If arguments are present, they are passed to set_source()
-        before filling.
+        before filling. Note that this source will be applied to further
+        fill()ings or stroke()ings or text() calls.
         """
         # Optionally set fill color, and save it.
         if source is not None:
@@ -386,9 +391,12 @@ class Drawing:
             cr.set_font_matrix(m)
         self.doStep(_font_rotate, degrees)
 
-    def image_surface(self, x, y, width, height, surface):
+    def image_surface(self, x, y, width, height, surface, mask=None):
         """Draw a given cairo.ImageSurface centered at (x, y), at the given
         width and height.
+
+        If you specify mask, it must be a cairo ImageSurface.
+        
         """
         # Calculate centering and scaling
         add_x, add_y = (0, 0)
@@ -407,10 +415,16 @@ class Drawing:
         self.scale(scale, scale)
 
         # Create and append the drawing function
-        def _image_surface(cr):
-            cr.set_source_surface(surface)
-            cr.paint()
-        self.doStep(_image_surface, x, y, width, height, surface)
+        if (mask):
+            def _image_surface(cr):
+                cr.set_source_surface(surface)
+                cr.mask_surface(mask)
+            self.doStep(_image_surface, x, y, width, height, surface, mask)
+        else:
+            def _image_surface(cr):
+                cr.set_source_surface(surface)
+                cr.paint()
+            self.doStep(_image_surface, x, y, width, height, surface, mask)
 
         self.restore()
     
@@ -949,31 +963,31 @@ class Drawing:
     ### ----------------------------------------------------------------
     
     
-    def font_slant(self, style):
-        """Set the font slant. Use one of:
-    
-           normal, italic, oblique
-        """
-        tr = {'italic': cairo.FONT_SLANT_ITALIC,
-              'normal': cairo.FONT_SLANT_NORMAL,
-              'oblique': cairo.FONT_SLANT_OBLIQUE}
-        def _font_slant(cr):
-            cr.set_slant(tr[style])
-        self.doStep(_font_slant, style)
-    # Alias
-    font_style = font_slant
+#    def font_slant(self, style):
+#        """Set the font slant. Use one of:
+#    
+#           normal, italic, oblique
+#        """
+#        tr = {'italic': cairo.FONT_SLANT_ITALIC,
+#              'normal': cairo.FONT_SLANT_NORMAL,
+#              'oblique': cairo.FONT_SLANT_OBLIQUE}
+#        def _font_slant(cr):
+#            cr.set_slant(tr[style])
+#        self.doStep(_font_slant, style)
+#    # Alias
+#    font_style = font_slant
 
 
-    def font_weight(self, weight):
-        """Set the font weight to one of:
-        
-            normal, bold
-        """
-        tr = {'normal': cairo.FONT_WEIGHT_NORMAL,
-              'bold': cairo.FONT_WEIGHT_BOLD}
-        def _font_weight(cr):
-            cr.set_font_weight(tr[weight])
-        self.doStep(_font_weight, weight)
+#    def font_weight(self, weight):
+#        """Set the font weight to one of:
+#        
+#            normal, bold
+#        """
+#        tr = {'normal': cairo.FONT_WEIGHT_NORMAL,
+#              'bold': cairo.FONT_WEIGHT_BOLD}
+#        def _font_weight(cr):
+#            cr.set_font_weight(tr[weight])
+#        self.doStep(_font_weight, weight)
 
 
     def font_family(self, family):
@@ -1077,6 +1091,53 @@ class Drawing:
 ### Exported functions
 ### --------------------------------------------------------------------
 
+
+# Cached masks, rendered only once.
+interlace_fields = None
+
+
+def interlace_drawings(draw1, draw2):
+    """Merge two drawings into one, using interlacing fields.
+
+    This method interlaces with the BOTTOM-FIRST field order.
+    """
+    global interlace_fields
+    
+    dr = Drawing(draw1.w, draw1.h)
+
+    # create masks (only once)
+    if (not interlace_fields):
+        fields = [0, 1]
+        for f in range(0,2):
+            img = cairo.ImageSurface(cairo.FORMAT_ARGB32, draw1.w, draw1.h)
+            cr = cairo.Context(img)
+            cr.set_antialias(cairo.ANTIALIAS_NONE)
+            cr.set_source_rgba(0.5, 0.5, 0.5, 1)
+            cr.set_line_width(1)
+
+            for x in range(0, draw1.h / 2):
+                # x*2 + f = top field first.., each two lines..
+                # -1  draw1.w +1 -> pour tre sur de couvrir aussi les bouts.
+                cr.move_to(-1, x*2 + f)
+                cr.line_to(draw1.w + 1, x*2 + f)
+                cr.stroke()
+
+            fields[f] = img
+        interlace_fields = fields
+    else:
+        fields = interlace_fields
+
+    # For bottom first, use fields[0] first, and fields[1] after.
+    # For top-first, use draw1 with fields[1] and draw2 with fields[0]
+    # paint first image
+    dr.image_surface(0,0,dr.w, dr.h, draw1.surface, fields[0])
+    # paint second image
+    dr.image_surface(0,0,dr.w, dr.h, draw2.surface, fields[1])
+
+    return dr
+
+
+
 def render(drawing, context, width, height):
     """Render a Drawing to the given cairo context at the given size.
     """
@@ -1123,6 +1184,58 @@ def display(drawing, width=None, height=None, fix_aspect=False):
     print commands.getoutput('display %s' % png_file)
 
 
+def write_ppm(drawing, pipe, width, height, workdir=None):
+    """Write image as a PPM file to a file-object
+
+      workdir - Unused in this context, just to have parallel parameters
+                with write_png().
+
+    Useful to pipe directly in ppmtoy4m and to pipe directly to mpeg2enc.
+    """
+    # Timing
+    start = time.time()
+    if (width, height) == drawing.size:
+        #print "Not re-rendering"
+        surface = drawing.surface
+    else:
+        surface = get_surface(width, height, 'image')
+        context = cairo.Context(surface)
+        render(drawing, context, width, height)
+        
+    buf = surface.get_data()
+    # Assumes surface is cairo.FORMAT_ARGB32
+    im = Image.frombuffer('RGBA', (surface.get_width(),  surface.get_height()),
+                          buf)
+    im = im.transpose(Image.FLIP_TOP_BOTTOM)
+    im.save(pipe, 'ppm')
+    
+    print "write_ppm took %s seconds" % (time.time() - start)
+    
+def write_png(drawing, pipe, width, height, workdir):
+    """Write image as a PPM file to a file-object
+
+    Useful to pipe directly in pngtopnm -mix -background=rgb:00/00/00 | ppmtoy4m | mpeg2enc.
+    """
+    # Timing
+    start = time.time()
+    if (width, height) == drawing.size:
+        #print "Not re-rendering"
+        surface = drawing.surface
+    else:
+        surface = get_surface(width, height, 'image')
+        context = cairo.Context(surface)
+        render(drawing, context, width, height)
+
+    surface.write_to_png('%s/tmp.png' % workdir)
+    im = Image.open('%s/tmp.png' % workdir)
+    im.load()
+    im.save(pipe, 'ppm')
+    
+    print "write_png took %s seconds" % (time.time() - start)
+
+
+
+
 def save_png(drawing, filename, width, height):
     """Saves a drawing to PNG, keeping alpha channel intact.
 
@@ -1136,10 +1249,9 @@ def save_png(drawing, filename, width, height):
         surface = get_surface(width, height, 'image')
         context = cairo.Context(surface)
         render(drawing, context, width, height)
-    print "Saving", filename
     surface.write_to_png(filename)
     print "save_png took %s seconds" % (time.time() - start)
-    
+
 
 def save_jpg(drawing, filename, width, height):
     """Saves a drawing to JPG, losing alpha channel information.
