@@ -1,32 +1,42 @@
 #! /usr/bin/env python
 # encode.py
 
-"""This module provides several video-encoding backends. It provides one
-high-level function:
+"""This module provides several backend for encoding video and audio.
 
-    encode(infile, outfile, format, tvsys, method)
+One high-level function is provided:
 
-For example:
+    encode(infile, outfile, format, tvsys, method, ...)
+
+where "..." is optional keyword arguments (described below). For example:
 
     encode('/video/foo.avi', '/video/bar.mpg', 'dvd', 'ntsc', 'mencoder')
 
-This will encode '/video/foo.avi' to NTSC DVD form using mencoder, saving
-the result as '/video/bar.mpg'.
+This will encode '/video/foo.avi' to NTSC DVD format using mencoder, saving
+the result as '/video/bar.mpg'. Keyword arguments may be used to further
+refine encoding behavior, for example:
 
-Each backend implements a top-level function:
+    encode('foo.avi', 'foo.mpg', 'dvd', 'pal', 'ffmpeg',
+           quality=7, interlace='bottom', ...)
 
-    ffmpeg_encode(source, target)
-    mencoder_encode(source, target)
-    mpeg2enc_encode(source, target)
+The supported keywords may vary by backend, but one of our goals should be to
+implement a set of commonly-used keywords in all the backends.
+
+Each backend has a top-level function:
+
+    ffmpeg_encode(source, target, ...)
+    mencoder_encode(source, target, ...)
+    mpeg2enc_encode(source, target, ...)
 
 The source and target are MediaFile objects from libtovid.media, containing
-profiles of the input and output videos.
+profiles of the input and output videos. Again, the "..." are optional keyword
+arguments.
 """
 
 __all__ = [\
     'encode',
     'get_encoder',
     'ffmpeg_encode',
+    'encode_audio',
     'mencoder_encode',
     'mpeg2enc_encode']
 
@@ -35,12 +45,20 @@ import math
 import copy
 import glob
 from libtovid.cli import Command, Pipe
-from libtovid.utils import float_to_ratio, ratio_to_float
+from libtovid.utils import float_to_ratio
 from libtovid.transcode import rip
 from libtovid.media import *
 from libtovid.standard import fps
 from libtovid import log
 
+_bitrate_limits = {\
+    'vcd': (1150, 1150),
+    'kvcd': (400, 4000),
+    'dvd-vcd': (400, 4000),
+    'svcd': (900, 2400),
+    'half-dvd': (600, 6000),
+    'dvd': (900, 9000)
+    }
 
 # --------------------------------------------------------------------------
 #
@@ -48,7 +66,8 @@ from libtovid import log
 #
 # --------------------------------------------------------------------------
 
-def encode(infile, outfile, format='dvd', tvsys='ntsc', method='ffmpeg'):
+def encode(infile, outfile, format='dvd', tvsys='ntsc', method='ffmpeg',
+           **kw):
     """Encode a multimedia file according to a target profile, saving the
     encoded file to outfile.
     
@@ -57,7 +76,10 @@ def encode(infile, outfile, format='dvd', tvsys='ntsc', method='ffmpeg'):
         format:  One of 'vcd', 'svcd', 'dvd' (case-insensitive)
         tvsys:   One of 'ntsc', 'pal' (case-insensitive)
         method:  Encoding backend: 'ffmpeg', 'mencoder', or 'mpeg2enc'
-
+        **kw:    Additional keyword arguments (name=value)
+    
+    The supported keyword arguments vary by encoding method. See the encoding
+    functions for what is available in each.
     """
     source = load_media(infile)
     # Add .mpg to outfile if not already present
@@ -65,13 +87,27 @@ def encode(infile, outfile, format='dvd', tvsys='ntsc', method='ffmpeg'):
         outfile += '.mpg'
     # Get an appropriate encoding target
     target = standard_media(format, tvsys)
-    # TODO: User-defined aspect ratio; hardcoded 4:3 for now
-    target = correct_aspect(source, target, '4:3')
     target.filename = outfile
+    # Set desired aspect ratio, or auto
+    if 'aspect' in kw:
+        target = correct_aspect(source, target, kw['aspect'])
+    else:
+        target = correct_aspect(source, target, 'auto')
     
-    # Get the appropriate encoding backend and encode
-    encoder = get_encoder(method)
-    encoder(source, target)
+    # Some friendly output
+    print "Source media:"
+    print source
+    print
+    print "Target media:"
+    print target
+    
+    # Get the appropriate encoding backend
+    encode_method = get_encoder(method)
+    # Evaluate high-level keywords
+    kw = eval_keywords(**kw)
+    # Encode!
+    encode_method(source, target, **kw)
+
 
 def get_encoder(backend):
     """Get an encoding function."""
@@ -82,6 +118,25 @@ def get_encoder(backend):
     elif backend == 'mpeg2enc':
         return mpeg2enc_encode
 
+def eval_keywords(**kw):
+    """Interpret keywords that affect other keywords, and return the result.
+    These are keywords that can be shared between multiple encoding backends.
+
+    Supported keywords:
+    
+        quality:    From 1 (lowest) to 10 (highest) video quality.
+                    Overrides 'quant' and 'vbitrate' keywords.
+        fit:        Size in MiB to fit output video to (overrides 'quality')
+                    Overrides 'quant' and 'vbitrate' keywords.
+
+    """
+    if 'quality' in kw:
+        kw['quant'] = 13-kw['quality']
+        max_bitrate = _bitrate_limits[target.format][1]
+        kw['vbitrate'] = kw['quality'] * max_bitrate / 10
+    if 'fit' in kw:
+        kw['quant'], kw['vbitrate'] = _fit(source, target, kw['fit'])
+    return kw
 
 
 # --------------------------------------------------------------------------
@@ -90,12 +145,24 @@ def get_encoder(backend):
 #
 # --------------------------------------------------------------------------
 
-def ffmpeg_encode(source, target):
+def ffmpeg_encode(source, target, **kw):
     """Encode a multimedia video using ffmpeg.
 
         source:  Input MediaFile
         target:  Output MediaFile
+        kw:      name=value keyword arguments
     
+    Supported keywords:
+    
+        quant:      Minimum quantization, from 1-31 (1 being fewest artifacts)
+        vbitrate:   Maximum video bitrate, in kilobits per second.
+        abitrate:   Audio bitrate, in kilobits per second
+        interlace:  'top' or 'bottom', to do interlaced encoding with
+                    top or bottom field first
+
+    For example:
+    
+        ffmpeg_encode(source, target, quant=4, vbitrate=7000)
     """
     # Build the ffmpeg command
     cmd = Command('ffmpeg')
@@ -103,13 +170,28 @@ def ffmpeg_encode(source, target):
     if target.format in ['vcd', 'svcd', 'dvd']:
         cmd.add('-tvstd', target.tvsys,
                 '-target', '%s-%s' % (target.tvsys, target.format))
-    
+
+    # Interpret keyword arguments
+    if 'quant' in kw:
+        cmd.add('-qmin', kw['quant'], '-qmax', 31)
+    if 'vbitrate' in kw:
+        cmd.add('-b', '%dk' % kw['vbitrate'])
+    if 'abitrate' in kw:
+        cmd.add('-ab', '%dk' % kw['abitrate'])
+    if 'interlace' in kw:
+        if kw['interlace'] == 'bottom':
+            cmd.add('-top', 0, '-flags', '+alt+ildct+ilme')
+        elif kw['interlace'] == 'top':
+            cmd.add('-top', 1, '-flags', '+alt+ildct+ilme')
+
+    # Convert frame rate and audio sampling rate
     cmd.add('-r', target.fps,
             '-ar', target.samprate)
+    
     # Convert scale/expand to ffmpeg's padding system
     if target.scale:
         cmd.add('-s', '%sx%s' % target.scale)
-    if target.expand > target.scale:
+    if target.expand != target.scale:
         e_width, e_height = target.expand
         s_width, s_height = target.scale
         h_pad = (e_width - s_width) / 2
@@ -126,7 +208,6 @@ def ffmpeg_encode(source, target):
     cmd.add('-y')
     cmd.add(target.filename)
     
-    # Run the command to do the encoding
     cmd.run()
 
 
@@ -136,12 +217,14 @@ def ffmpeg_encode(source, target):
 #
 # --------------------------------------------------------------------------
 
-def mencoder_encode(source, target):
+def mencoder_encode(source, target, **kw):
     """Encode a multimedia video using mencoder.
 
         source:  Input MediaFile
         target:  Output MediaFile
+        kw:      name=value keyword arguments
     
+    Supported keywords:
     """
 
     # Build the mencoder command
@@ -179,7 +262,7 @@ def mencoder_encode(source, target):
         else:
             audiofile = '%s.ac3' % target.filename
         encode_audio(source, audiofile, target)
-        # TODO: make this work, it,s still not adding the ac3 file correctly.
+        # TODO: make this work, it's still not adding the ac3 file correctly.
         cmd.add('-audiofile', audiofile)
 
     # Video codec
@@ -204,7 +287,7 @@ def mencoder_encode(source, target):
         lavcopts += ':aspect=16/9'
     else:
         lavcopts += ':aspect=4/3'
-    # Put all lavcopts together
+    # Pass all lavcopts together
     cmd.add('-lavcopts', lavcopts)
 
     # FPS
@@ -217,11 +300,10 @@ def mencoder_encode(source, target):
     if target.scale:
         vfilter = 'scale=%s:%s' % target.scale
         # Expand is not done unless also scaling
-        if target.expand > target.scale:
+        if target.expand != target.scale:
             vfilter += ',expand=%s:%s' % target.expand
         cmd.add('-vf', vfilter)
 
-    # Run the command to do the encoding
     cmd.run()
 
 # --------------------------------------------------------------------------
@@ -231,7 +313,7 @@ def mencoder_encode(source, target):
 # --------------------------------------------------------------------------
 
 
-def mpeg2enc_encode(source, target):
+def mpeg2enc_encode(source, target, **kw):
     """Encode a multimedia video using mplayer|yuvfps|mpeg2enc.
     
         source:  Input MediaFile
@@ -288,15 +370,16 @@ def encode_audio(source, audiofile, target):
     else:
         # Add silence the length of source length
         ln = source.length
+        #print "Source.length: %f" % ln
         if ln < 4:
             # Minimum 4 secs :)
             ln = 4.0
         cmd.add('-f', 's16le', '-i', '/dev/zero', '-t', '%f' % ln)
-    cmd.add_raw('-vn -ac 2 -ab 224')
+    cmd.add('-vn', '-ac', '2', '-ab', '224')
     cmd.add('-ar', target.samprate)
     cmd.add('-acodec', target.acodec)
     cmd.add('-y', audiofile)
-    # Run the command to encode the audio
+
     cmd.run()
 
 
@@ -353,31 +436,6 @@ def encode_video(source, yuvfile, videofile, target):
     pipe.run()
 
 
-def encode_audio(source, audiofile, target):
-    """Encode an audio stream to AC3 or MP2 format.
-    
-        source:    Input MediaFile
-        audiofile: File to put encoded audio in
-        target:    Output MediaFile
-        
-    """
-    cmd = Command('ffmpeg')
-    # If source file has audio, encode it
-    if source.has_audio:
-        cmd.add('-i', source.filename)
-    # Otherwise, generate 4-second silence
-    else:
-        cmd.add('-f', 's16le', '-i', '/dev/zero', '-t', '4')
-    # Add other necessary qualifiers
-    cmd.add('-ac', '2',
-            '-ab', '224',
-            '-ar', target.samprate,
-            '-acodec', target.acodec,
-            '-y', audiofile)
-    # Run the command to encode the audio
-    cmd.run()
-
-
 def mplex_streams(vstream, astream, target):
     """Multiplex audio and video stream files to the given target.
     
@@ -410,18 +468,28 @@ def mplex_streams(vstream, astream, target):
 #
 # --------------------------------------------------------------------------
 
-def encode_frames(imagedir, outfile, format, tvsys, aspect):
+def encode_frames(imagedir, outfile, format, tvsys, aspect, interlaced=False):
     """Convert an image sequence in the given directory to match a target
     MediaFile, putting the output stream in outfile.
 
-        imagedir:  Directory containing images (and only images)
-        outfile:   Filename for output.
-        tvsys:     TVsys desired for output (to deal with FPS)
-        aspect:    Aspect ratio ('4:3', '16:9')
+        imagedir:   Directory containing images (and only images)
+        outfile:    Filename for output.
+        tvsys:      TVsys desired for output (to deal with FPS)
+        aspect:     Aspect ratio ('4:3', '16:9')
+        interlaced: Frames are interlaced material.
         
     Currently supports JPG and PNG images; input images must already be
     at the desired target resolution.
     """
+    #
+    # Trying to implement interlaced in encoding failed with:
+    #    ++ WARN: [mpeg2enc] Frame height won't split into two equal field pictures...
+    #    ++ WARN: [mpeg2enc] forcing encoding as progressive video
+    # in all cases. Setting -Ip -L1 with pnv2yuv and -I2 in mpeg2enc, etc..
+    # The option is still there, so that compatibility continues, but it
+    # currently does nothing.
+    #
+    
     # Use absolute path name
     imagedir = os.path.abspath(imagedir)
     print "Creating video stream from image sequence in %s" % imagedir
@@ -442,18 +510,25 @@ def encode_frames(imagedir, outfile, format, tvsys, aspect):
     # Use jpeg2yuv/png2yuv to stream images
     if extension == 'jpg':
         jpeg2yuv = Command('jpeg2yuv')
-        jpeg2yuv.add('-Ip',
-                     '-f', '%.3f' % fps(tvsys),
+        
+        jpeg2yuv.add('-Ip') # Progressive
+            
+        jpeg2yuv.add('-f', '%.3f' % fps(tvsys),
                      '-j', '%s/%%08d.%s' % (imagedir, extension))
         pipe.add(jpeg2yuv)
     elif extension == 'png':
         #ls = Command('sh', '-c', 'ls %s/*.png' % imagedir)
         #xargs = Command('xargs', '-n1', 'pngtopnm')
         png2yuv = Command('png2yuv')
-        png2yuv.add('-Ip',
-                    '-f', '%.3f' % fps(tvsys),
+
+        png2yuv.add('-Ip') # Progressive
+            
+
+        png2yuv.add('-f', '%.3f' % fps(tvsys),
                     '-j', '%s/%%08d.png' % (imagedir))
+        
         pipe.add(png2yuv)
+
         #pipe.add(ls, xargs, png2yuv)
         #cmd += 'pnmtoy4m -Ip -F %s %s/*.png' % standard.fpsratio(tvsys)
 
@@ -461,6 +536,15 @@ def encode_frames(imagedir, outfile, format, tvsys, aspect):
     
     # Pipe image stream into mpeg2enc to encode
     mpeg2enc = Command('mpeg2enc')
+
+    # Anyways.
+    mpeg2enc.add('-I 0') # Progressive
+
+    # Bottom-first, determined in render/drawing.py::interlace_drawings()
+    if (interlaced):
+        mpeg2enc.add('--playback-field-order', 'b')  # Bottom-first field order
+        # conforming to drawing.py::interlace_drawings()
+    
     mpeg2enc.add('-v', '0',
                  '-q' '3',
                  '-o' '%s' % outfile)
@@ -481,3 +565,38 @@ def encode_frames(imagedir, outfile, format, tvsys, aspect):
 
     pipe.run(capture=True)
     pipe.get_output() # and wait :)
+
+
+def _fit(source, target, fit_size):
+    """Return video (quantization, bitrate) to fit a video into a given size.
+    
+        source:   MediaFile input (the video being encoded)
+        target:   MediaFile output (desired target profile)
+        fit_size: Desired encoded file size, in MiB
+
+    """
+    assert isinstance(source, MediaFile)
+    assert isinstance(target, MediaFile)
+    fit_size = float(fit_size)
+    mpeg_overhead = fit_size / 100
+    aud_vid_size = fit_size - mpeg_overhead
+    audio_size = float(source.length * target.abitrate) / (8*1024)
+    video_size = aud_vid_size - audio_size
+    vid_bitrate = int(video_size*8*1024 / source.length)
+
+    print "Length:", source.length, "seconds"
+    print "Overhead:", mpeg_overhead, "MiB"
+    print "Audio:", audio_size, "MiB"
+    print "Video:", video_size, "MiB"
+    print "VBitrate:", vid_bitrate, "kbps"
+
+    # Keep bitrates sane for each disc format
+    lower, upper = _bitrate_limits[target.format]
+    quant = 3
+    if vid_bitrate < lower:
+        return (quant, lower)
+    elif vid_bitrate > upper:
+        return (quant, upper)
+    else:
+        return (quant, vid_bitrate)
+
